import java.lang.reflect.ParameterizedType;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class Checkbook {

    //blind us with your Java eloquence....

    private List<Transaction> debitTransactions;
    private List<Transaction> creditTransactions;

    Payee owner;
    double cashAmount;

    Checkbook(){
        new Checkbook(0);
    }

    Checkbook(double cashAmount){
        new Checkbook(new Payee(),cashAmount);
    }

    Checkbook(Payee owner, double cashAmount){
        this.owner = owner;
        this.cashAmount = cashAmount;
        debitTransactions = new ArrayList<>();
        creditTransactions = new ArrayList<>();
    }

    public boolean add(Transaction element){
        if (element.getType() == null) return false;
        if (element.getType().equals(TransactionType.DEBIT))
            return debitTransactions.add(element);
        else return creditTransactions.add(element);
    }

    public int size(){
        return debitTransactions.size() + creditTransactions.size();
    }

    public boolean isEmpty(){
        return debitTransactions.isEmpty() && creditTransactions.isEmpty();
    }

    public boolean contains(Transaction element){
        return debitTransactions.contains(element) || creditTransactions.contains(element);
    }

    public boolean remove(Transaction element){
        return debitTransactions.remove(element) || creditTransactions.remove(element);
    }

    public doubleRater iterator(){
        return new doubleRater();
    }

    public Transaction[] toArray(){
        return Stream.concat(debitTransactions.stream(),creditTransactions.stream()).toArray(Transaction[]::new);
    }

    public Transaction[] toArray(Transaction[] array){
        List<Transaction> list = Stream.concat(debitTransactions.stream(),creditTransactions.stream())
                .collect(Collectors.toList());
        return list.toArray(array);
    }

    public double checkbookValue(){
        double sum = debitTransactions.stream().mapToDouble(Transaction::getAmount).sum();
        sum += creditTransactions.stream().mapToDouble(Transaction::getAmount).sum();
        return sum;
    }

    public Transaction[] getTransactionsForDate(LocalDate aDate){
        return Arrays.stream(toArray()).filter(e -> e.getDate().equals(aDate)).toArray(Transaction[]::new);
    }

    public Transaction[] getAllDebitTransactions(){
        return debitTransactions.toArray(new Transaction[0]);
    }

    public Transaction[] getAllCreditTransactions(){
        return creditTransactions.toArray(new Transaction[0]);
    }

    public boolean containsAll(Collection c){
        return Stream.concat(debitTransactions.stream(),creditTransactions.stream())
                .collect(Collectors.toList()).containsAll(c);
    }

    public boolean addAll(Collection c){
        for (Object o : c){
            if (!o.getClass().getSimpleName().equals("Transaction"))
                return false;
            add((Transaction) o);
        }
        return true;
    }

    public boolean removeAll(Collection c){
        int size = size();
        for (Object o : c)
            remove((Transaction) o);
        return size != size();
    }

    public void clear(){
        debitTransactions.clear();
        creditTransactions.clear();
    }

    private class doubleRater implements Iterator<Transaction> {
        private final List<Iterator<Transaction>> list;
        private final Iterator<Iterator<Transaction>> it;
        private Iterator<Transaction> current;

        doubleRater(){
            list = new ArrayList<>();
            list.add(debitTransactions.iterator());
            list.add(creditTransactions.iterator());
            it = list.iterator();
            current = it.next();
        }


        @Override
        public boolean hasNext() {
            checkIterator();
            return current.hasNext();
        }

        @Override
        public Transaction next() {
            checkIterator();
            return current.next();
        }

        @Override
        public void remove() {
            current.remove();
        }

        private void checkIterator(){
            if (!current.hasNext() && it.hasNext())
                current = it.next();
        }
    }
}
