import java.time.LocalDate;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicReference;

public class Transaction implements AccountBook{
    private final AtomicReference<Integer> id = new AtomicReference<Integer>();
    private final AtomicReference<LocalDate> date = new AtomicReference<LocalDate>(LocalDate.now());
    private String memo;
    private Payee payee;
    private TransactionType typee; // Credit (reduces the checkbook), Debit (increases the checkbook)
    private Double amount;

    public Transaction(String memo, Payee payee, TransactionType typee, Double amount) {
        this.typee = typee;
        this.memo = memo;
        this.payee = payee;
        this.amount = amount;
    }

    public Integer getId() {
        return id.get();
    }

    public LocalDate getDate() {
        return date.get();
    }

    public TransactionType getType() {
        return typee;
    }

    public String getMemo() {
        return memo;
    }

    public void setMemo(String memo) {
        this.memo = memo;
    }

    public Payee getPayee() {
        return payee;
    }

    public void setPayee(Payee payee) {
        this.payee = payee;
    }

    public Double getAmount() {
        if (this.typee == TransactionType.DEBIT) {
            return amount;
        }
        return (-1.0 * amount);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Transaction that = (Transaction) o;
        return Objects.equals(id.get(), that.id.get()) &&
                Objects.equals(date.get(), that.date.get()) &&
                Objects.equals(memo, that.memo) &&
                Objects.equals(payee, that.payee) &&
                typee == that.typee &&
                Objects.equals(amount, that.amount);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, date, memo, payee, typee, amount);
    }
}
