import org.junit.Assert;
import org.junit.Test;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import static org.junit.Assert.*;

public class CheckbookTest {

    Checkbook checkbook;
    Payee owner;
    Transaction transaction;

    @org.junit.Before
    public void setUp() throws Exception {
        checkbook = new Checkbook(owner,100);
        transaction = new Transaction("-",owner,TransactionType.DEBIT,1.0);
    }

    @org.junit.After
    public void tearDown() throws Exception {
    }

    @Test
    public void testIsEmpty(){
        Assert.assertTrue(checkbook.isEmpty());
    }

    @Test
    public void testAdd(){
        checkbook.add(transaction);
        Assert.assertEquals(1,checkbook.size());
    }

    @Test
    public void testContains(){
        checkbook.add(transaction);
        Assert.assertTrue(checkbook.contains(transaction));
    }

    @Test
    public void testRemove(){
        checkbook.add(transaction);
        Assert.assertTrue(checkbook.remove(transaction));
    }

    @Test
    public void testIterator(){
        Assert.assertNotNull(checkbook.iterator());
    }

    @Test
    public void testIteratorIterates(){
        createTransactions();

        Iterator<Transaction> it = checkbook.iterator();

        double sum = 0;
        while(it.hasNext())
            sum += it.next().getAmount();

        Assert.assertEquals(8.0,sum);
    }

    @Test
    public void testToArray(){
        createTransactions();

        Transaction[] transactionsArray = checkbook.toArray();
        assertEquals(8, transactionsArray.length);
    }

    @Test
    public void testToArray2(){
        Transaction[] t = new Transaction[]{transaction};
        createTransactions();

        Transaction[] transactionsArray = checkbook.toArray(t);
        assertEquals(8, transactionsArray.length);
    }

    @Test
    public void testCheckbookValue(){
        createTransactions();
        Assert.assertEquals(8.0,checkbook.checkbookValue());
    }

    @Test
    public void testGetTransactionsForDate(){
        createTransactions();
        Assert.assertEquals(8,checkbook.getTransactionsForDate(LocalDate.of(2018,12,8)).length);
    }

    @Test
    public void testGetAllDebitTransactions(){
        createTransactions();
        Assert.assertEquals(4,checkbook.getAllDebitTransactions().length);
    }

    @Test
    public void testGetAllCreditTransactions(){
        createTransactions();
        Assert.assertEquals(4,checkbook.getAllCreditTransactions().length);
    }

    @Test
    public void testContainsAll(){
        createTransactions();
        List<Transaction> list = new ArrayList<>();
        Transaction t1 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
        Transaction t2 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
        list.add(t1);
        list.add(t2);

        Assert.assertTrue("doesn't contain", checkbook.containsAll(list));

    }

    @Test
    public void testAddAll(){
        createTransactions();
        List<Transaction> list = new ArrayList<>();
        Transaction t1 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
        Transaction t2 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
        list.add(t1);
        list.add(t2);

        Assert.assertTrue("doesn't contain", checkbook.addAll(list));
    }

    @Test
    public void testRemoveAll(){
        createTransactions();
        List<Transaction> list = new ArrayList<>();
        Transaction t1 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
        Transaction t2 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
        list.add(t1);
        list.add(t2);

        Assert.assertTrue(checkbook.removeAll(list));
    }

    @Test
    public void testClear(){
        createTransactions();
        checkbook.clear();
        Assert.assertEquals(0,checkbook.size());
    }

    public void createTransactions(){
        owner = new Payee();
        owner.setGivenName("Will");
        owner.setFamilyName("Smith");

        Payee person = new Payee();
        person.setFamilyName("Jackson");
        person.setGivenName("Jet");

        for (int i=0; i<4; i++) {
            Transaction t1 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
            Transaction t2 = new Transaction("--", person, TransactionType.CREDIT, 2.0);

            checkbook.add(t1);
            checkbook.add(t2);
        }
    }

}