import org.junit.Assert;
import org.junit.Test;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import static org.junit.Assert.assertEquals;

public class GenericAccountTest {
    GenericAccount<Transaction> genAccount;
    Payee owner;
    Transaction transaction;

    @org.junit.Before
    public void setUp() throws Exception {
        genAccount = new GenericAccount<>(owner,100);
        transaction = new Transaction("-",owner,TransactionType.DEBIT,1.0);
    }

    @org.junit.After
    public void tearDown() throws Exception {
    }

    @Test
    public void testIsEmpty(){
        Assert.assertTrue(genAccount.isEmpty());
    }

    @Test
    public void testAdd(){
        genAccount.add(transaction);
        Assert.assertEquals(1,genAccount.size());
    }

    @Test
    public void testContains(){
        genAccount.add(transaction);
        Assert.assertTrue(genAccount.contains(transaction));
    }

    @Test
    public void testRemove(){
        genAccount.add(transaction);
        Assert.assertTrue(genAccount.remove(transaction));
    }

    @Test
    public void testIterator(){
        Assert.assertNotNull(genAccount.iterator());
    }

    @Test
    public void testIteratorIterates(){
        createTransactions();

        Iterator<Transaction> it = genAccount.iterator();

        double sum = 0;
        while(it.hasNext())
            sum += it.next().getAmount();

        Assert.assertEquals(8.0,sum);
    }

    @Test
    public void testToArray(){
        createTransactions();

        Transaction[] transactionsArray = genAccount.toArray();
        assertEquals(8, transactionsArray.length);
    }

    @Test
    public void testToArray2(){
        Transaction[] t = new Transaction[]{transaction};
        createTransactions();

        Transaction[] transactionsArray = genAccount.toArray(t);
        assertEquals(8, transactionsArray.length);
    }

    @Test
    public void testCheckbookValue(){
        createTransactions();
        Assert.assertEquals(8.0,genAccount.checkbookValue());
    }

    @Test
    public void testGetTransactionsForDate(){
        createTransactions();
        Assert.assertEquals(8,genAccount.getTransactionsForDate(LocalDate.of(2018,12,8)).length);
    }

    @Test
    public void testContainsAll(){
        createTransactions();
        List<Transaction> list = new ArrayList<>();
        Transaction t1 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
        Transaction t2 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
        list.add(t1);
        list.add(t2);

        Assert.assertTrue("doesn't contain", genAccount.containsAll(list));

    }

    @Test
    public void testAddAll(){
        createTransactions();
        List<Transaction> list = new ArrayList<>();
        Transaction t1 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
        Transaction t2 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
        list.add(t1);
        list.add(t2);

        Assert.assertTrue("doesn't contain", genAccount.addAll(list));
    }

    @Test
    public void testRemoveAll(){
        createTransactions();
        List<Transaction> list = new ArrayList<>();
        Transaction t1 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
        Transaction t2 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
        list.add(t1);
        list.add(t2);

        Assert.assertTrue(genAccount.removeAll(list));
    }

    @Test
    public void testClear(){
        createTransactions();
        genAccount.clear();
        Assert.assertEquals(0,genAccount.size());
    }

    public void createTransactions(){
        owner = new Payee();
        owner.setGivenName("Will");
        owner.setFamilyName("Smith");

        Payee person = new Payee();
        person.setFamilyName("Jackson");
        person.setGivenName("Jet");

        for (int i=0; i<4; i++) {
            Transaction t1 = new Transaction("--", owner, TransactionType.DEBIT, 4.0);
            Transaction t2 = new Transaction("--", person, TransactionType.CREDIT, 2.0);

            genAccount.add(t1);
            genAccount.add(t2);
        }
    }
}
